# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from ai_flow.graph.graph import Graph
from ai_flow.graph.channel import Channel
from ai_flow.ai_graph.ai_node import AINode
from typing import Dict, List, Text, Optional, Union
from ai_flow.ai_graph.data_edge import DataEdge
from ai_flow.workflow.job_config import JobConfig
from ai_flow.context.workflow_config_loader import current_workflow_config
from ai_flow.context.job_context import current_job_name


class AIGraph(Graph):
    """
    An AIGraph represents a user-defined workflow.
    AIGraph consists of AINodes and edges. The edges are of two kinds: the data edge(:class:`~ai_flow.ai_graph.data_edge.DataEdge`) between
    :class:`~ai_flow.ai_graph.ai_node.AINode` in a job and the control edge
    (:class:`~ai_flow.workflow.control_edge.ControlEdge`) between jobs.
    """

    def __init__(self) -> None:
        super().__init__()
        self.nodes: Dict[Text, AINode] = {}

    def add_node(self, node: AINode):
        """
        Adds an :class:`~ai_flow.ai_graph.ai_node.AINode` to AIGraph.

        :param node: :class:`~ai_flow.ai_graph.ai_node.AINode` to add in the graph.
        """
        if current_workflow_config() is not None \
                and current_job_name() is not None \
                and current_job_name() in current_workflow_config().job_configs:
            node.config = current_workflow_config().job_configs.get(current_job_name())
        self.nodes[node.node_id] = node

    def get_node_by_id(self, node_id: Text) -> Optional[AINode]:
        """
        Returns the node whose `node_id` field is node_id.

        :param node_id: id of the :class:`~ai_flow.ai_graph.ai_node.AINode`.
        :return :class:`~ai_flow.ai_graph.ai_node.AINode`.
        """
        if node_id in self.nodes:
            return self.nodes[node_id]
        else:
            return None

    def add_channel(self, node_id: Text, channel: Channel):
        """
        Adds a data edge to AIGraph.

        :param node_id: node_id of the data receiving node.
        :param channel: An output of the data sending node.
        """
        edge = DataEdge(destination=node_id, source=channel.node_id, port=channel.port)
        self.add_edge(node_id=node_id, edge=edge)


__current_ai_graph__ = AIGraph()


def current_graph() -> AIGraph:
    """
    Returns the current AIGraph.

    :return :class:`~ai_flow.ai_graph.ai_graph.AIGraph`.
    """
    return __current_ai_graph__


def add_ai_node_to_graph(node, inputs: Union[None, Channel, List[Channel]]):
    current_graph().add_node(node)
    if isinstance(inputs, Channel):
        current_graph().add_channel(node_id=node.node_id, channel=inputs)

    elif isinstance(inputs, List):
        for c in inputs:
            current_graph().add_channel(node_id=node.node_id, channel=c)


class AISubGraph(AIGraph):
    """
    It consists of a set of ai nodes(:class:`~ai_flow.ai_graph.ai_node.AINode`),
    all :class:`~ai_flow.ai_graph.ai_node.AINode` in sub-graph share the same job configuration
    (:class:`~ai_flow.workflow.job_config.JobConfig`).
    """

    def __init__(self,
                 config: JobConfig,
                 ) -> None:
        """
        :param config: :class:`~ai_flow.workflow.job_config.JobConfig` of the job represented by the sub-graph.
        """
        super().__init__()
        self.config: JobConfig = config
        self.edges: Dict[Text, List[DataEdge]] = {}

    def add_node(self, node: AINode):
        """
        Adds an :class:`~ai_flow.ai_graph.ai_node.AINode` to AISubGraph.

        :param node: :class:`~ai_flow.ai_graph.ai_node.AINode` to add in the sub-graph.
        """
        self.nodes[node.node_id] = node



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import enum
from typing import Text, List, Dict

from ai_flow.meta.dataset_meta import DatasetMeta
from ai_flow.graph.node import Node
from ai_flow.workflow.job import JobConfig
from ai_flow.util import serialization_utils
from ai_flow.graph.channel import Channel


class AINode(Node):
    """
    AINode is the basic component of ai graph(:class:`~ai_flow.ai_graph.ai_graph.AIGraph`),
    and there are edges(:class:`ai_flow.ai_graph.data_edge.DataEdge`) between the AINodes in
    :class:`~ai_flow.ai_graph.ai_graph.AIGraph`.
    """
    def __init__(self,
                 processor: object = None,
                 name: Text = None,
                 output_num: int = 1,
                 config: JobConfig = None,
                 node_type: Text = 'AINode',
                 **kwargs) -> None:
        """
        :param processor: The user defined function. Users can implement their own logic.
                          Different job types define different types of processors.
                          The user needs to implement the corresponding processor according to the specific job type.
        :param name: The name of the AINode.
        :param output_num: The number of output `:class:`~ai_flow.graph.channel.Channel` of the AINode
        :param config: The job config(:class:`~ai_flow.workflow.job_config.JobConfig`) of the AINode.
        :param node_type: User-defined node type.
        :param kwargs: User-defined variable parameters.
        """
        super().__init__(properties=None,
                         name=name,
                         output_num=output_num)
        self.processor: bytes = serialization_utils.serialize(processor)
        self.config: JobConfig = config
        self.node_config: Dict = kwargs
        self.node_config['name'] = name
        self.node_config['node_type'] = node_type

    def outputs(self) -> List[Channel]:
        if self.output_num > 0:
            result = []
            for i in range(self.output_num):
                result.append(Channel(node_id=self.node_id, port=i))
            return result
        else:
            return None

    def get_processor(self) -> object:
        """
        Return the processor object of the node.

        :return: The processor object.
        """
        if self.processor is None:
            return None
        else:
            return serialization_utils.deserialize(self.processor)

    def node_type(self):
        """
        Returns the type of the node.

        :return: type string.
        """
        return self.node_config.get('node_type')

    def name(self):
        """
        Returns the name of the node.

        :return: name string.
        """
        return self.node_config.get('name')


class ReadDatasetNode(AINode):
    """
    Represents the AINode that reads the dataset.
    """
    def __init__(self,
                 dataset: DatasetMeta,
                 processor: object = None,
                 name: Text = None,
                 config: JobConfig = None,
                 node_type: Text = 'ReadDataNode', **kwargs) -> None:
        """
        :param dateset: Meta information of the dataset that will be read.
        :param processor: The user defined function. Users can implement their own logic.
        :param name: The name of the ReadDatasetNode.
        :param config: The job config(:class:`~ai_flow.workflow.job_config.JobConfig`) of the AINode.
        :param node_type: User-defined node type. Default value is 'ReadDataNode'
        :param kwargs: User-defined variable parameters.
        """
        super().__init__(processor, name, 1, config, node_type, **kwargs)
        if dataset is None:
            raise Exception('dataset can not be None!')
        self.node_config['dataset'] = dataset

    def dataset(self):
        return self.node_config.get('dataset')


class WriteDatasetNode(AINode):
    """
    Represents the AINode that write the dataset.
    """
    def __init__(self,
                 dataset: DatasetMeta,
                 processor: object = None,
                 name: Text = None,
                 config: JobConfig = None,
                 node_type: Text = 'WriteDataNode', **kwargs) -> None:
        """
        :param dateset: Meta information of the dataset that will be written.
        :param processor: The user defined function. Users can implement their own logic.
        :param name: The name of the WriteDatasetNode.
        :param config: The job config(:class:`~ai_flow.workflow.job_config.JobConfig`) of the AINode.
        :param node_type: User-defined node type. Default value is 'WriteDataNode'
        :param kwargs: User-defined variable parameters.
        """
        super().__init__(processor, name, 0, config, node_type, **kwargs)
        if dataset is None:
            raise Exception('dataset can not be None!')
        self.node_config['dataset'] = dataset

    def dataset(self):
        """
        Returns metadata of the dataset.

        :return: :class:`ai_flow.meta.dataset_meta.DatasetMeta` of the WriteDatasetNode.
        """
        return self.node_config.get('dataset')




#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Client for all the API clients."""
import requests


class Client:
    """Base API client for all API clients."""

    def __init__(self, api_base_url, auth=None, session=None):
        self._api_base_url = api_base_url
        self._session: requests.Session = session or requests.Session()
        if auth:
            self._session.auth = auth

    def trigger_dag(self, dag_id, run_id=None, conf=None, execution_date=None):
        """Create a dag run for the specified dag.

        :param dag_id:
        :param run_id:
        :param conf:
        :param execution_date:
        :return:
        """
        raise NotImplementedError()

    def delete_dag(self, dag_id):
        """Delete all DB records related to the specified dag.

        :param dag_id:
        """
        raise NotImplementedError()

    def get_pool(self, name):
        """Get pool.

        :param name: pool name
        """
        raise NotImplementedError()

    def get_pools(self):
        """Get all pools."""
        raise NotImplementedError()

    def create_pool(self, name, slots, description):
        """Create a pool.

        :param name: pool name
        :param slots: pool slots amount
        :param description: pool description
        """
        raise NotImplementedError()

    def delete_pool(self, name):
        """Delete pool.

        :param name: pool name
        """
        raise NotImplementedError()

    def get_lineage(self, dag_id: str, execution_date: str):
        """
        Return the lineage information for the dag on this execution date
        :param dag_id:
        :param execution_date:
        :return:
        """
        raise NotImplementedError()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import unittest
from typing import Text
from ai_flow.plugin_interface.blob_manager_interface import BlobConfig, BlobManagerFactory, BlobManager


class MockBlockManager(BlobManager):
    def __init__(self, config):
        super().__init__(config)

    def upload_project(self, workflow_snapshot_id: Text, project_path: Text) -> Text:
        return 'upload'

    def download_project(self, workflow_snapshot_id, remote_path: Text, local_path: Text = None) -> Text:
        return 'download'


class TestBlobManager(unittest.TestCase):

    def test_blob_manager_factory(self):
        config = {
            'blob_manager_class': 'ai_flow.test.plugin_interface.test_blob_manager.MockBlockManager'
        }
        blob_config = BlobConfig(config)
        blob_manager = BlobManagerFactory.create_blob_manager(blob_config.blob_manager_class(),
                                                              blob_config.blob_manager_config())
        uploaded_path = blob_manager.upload_project('1', None)
        self.assertEqual('upload', uploaded_path)

        downloaded_path = blob_manager.download_project('1', uploaded_path)
        self.assertEqual('download', downloaded_path)


if __name__ == '__main__':
    unittest.main()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import ast
from ai_flow.meta.artifact_meta import ArtifactMeta
from ai_flow.meta.dataset_meta import DatasetMeta, DataType, Schema
from ai_flow.meta.model_relation_meta import ModelRelationMeta, ModelVersionRelationMeta, \
    create_model_version_relation
from ai_flow.meta.project_meta import ProjectMeta
from ai_flow.meta.workflow_meta import WorkflowMeta


class ResultToMeta:
    @staticmethod
    def result_to_dataset_meta(dataset_result) -> DatasetMeta:
        properties = dataset_result.properties
        if properties is not None:
            properties = ast.literal_eval(properties)
        name_list = dataset_result.name_list
        if name_list is not None:
            name_list = ast.literal_eval(name_list)
        type_list = dataset_result.type_list
        if type_list is not None:
            type_list = ast.literal_eval(type_list)
            data_type_list = []
            for data_type in type_list:
                data_type_list.append(DataType(data_type))
        else:
            data_type_list = None
        schema = Schema(name_list=name_list, type_list=data_type_list)
        return DatasetMeta(uuid=dataset_result.uuid, name=dataset_result.name,
                           data_format=dataset_result.format,
                           description=dataset_result.description,
                           uri=dataset_result.uri,
                           create_time=dataset_result.create_time,
                           update_time=dataset_result.update_time, schema=schema,
                           properties=properties, catalog_name=dataset_result.catalog_name,
                           catalog_type=dataset_result.catalog_type, catalog_database=dataset_result.catalog_database,
                           catalog_connection_uri=dataset_result.catalog_connection_uri,
                           catalog_table=dataset_result.catalog_table)

    @staticmethod
    def result_to_project_meta(project_result) -> ProjectMeta:
        properties = project_result.properties
        if properties is not None:
            properties = ast.literal_eval(properties)
        return ProjectMeta(uuid=project_result.uuid, name=project_result.name, uri=project_result.uri,
                           properties=properties)

    @staticmethod
    def result_to_artifact_meta(artifact_result) -> ArtifactMeta:
        properties = artifact_result.properties
        if properties is not None:
            properties = ast.literal_eval(properties)
        return ArtifactMeta(uuid=artifact_result.uuid, name=artifact_result.name,
                            artifact_type=artifact_result.artifact_type,
                            description=artifact_result.description, uri=artifact_result.uri,
                            create_time=artifact_result.create_time,
                            update_time=artifact_result.update_time, properties=properties)

    @staticmethod
    def result_to_model_relation_meta(model_result) -> ModelRelationMeta:
        return ModelRelationMeta(uuid=model_result.uuid, name=model_result.name, project_id=model_result.project_id)

    @staticmethod
    def result_to_model_version_relation_meta(model_version_result) -> ModelVersionRelationMeta:
        return create_model_version_relation(version=model_version_result.version,
                                             model_id=model_version_result.model_id,
                                             project_snapshot_id=model_version_result.project_snapshot_id)

    @staticmethod
    def result_to_workflow_meta(workflow_result) -> WorkflowMeta:
        properties = workflow_result.properties
        if properties is not None:
            properties = ast.literal_eval(properties)
        return WorkflowMeta(name=workflow_result.name,
                            project_id=workflow_result.project_id,
                            properties=properties,
                            create_time=workflow_result.create_time,
                            update_time=workflow_result.update_time,
                            uuid=workflow_result.uuid)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from enum import Enum
from typing import Text, List

from ai_flow.graph.edge import Edge
from ai_flow.util.json_utils import Jsonable
from notification_service.base_notification import UNDEFINED_EVENT_TYPE, DEFAULT_NAMESPACE


class ConditionType(str, Enum):
    """
    MEET_ALL: The action will be triggered when and only all the EventConditions are met.
    MEET_ANY: The action will be triggered when any of the EventConditions is met.
    """
    MEET_ALL = "MEET_ALL"
    MEET_ANY = "MEET_ANY"


class JobAction(str, Enum):
    """
    START: Start the job.
    RESTART: If the job is running, stop it first and then start it. If the job is not running,just start it.
    STOP: Stop the job.
    NONE: Do nothing.
    """
    START = "START"
    RESTART = "RESTART"
    STOP = "STOP"
    NONE = "NONE"


class EventLife(str, Enum):
    """
    ONCE: The event value will be used only once.
    REPEATED: The event value will be used repeated.
    """
    ONCE = "ONCE"
    REPEATED = "REPEATED"


class ValueCondition(str, Enum):
    """
    EQUALS: The condition that notification service updates a value which equals to the event value.
    UPDATED: The condition that notification service has a update operation on the event key which event
            value belongs.
    """
    EQUALS = "EQUALS"
    UPDATED = "UPDATE"


class EventMeetConfig(Jsonable):
    def __init__(self,
                 event_key: Text,
                 event_value: Text,
                 event_type: Text = UNDEFINED_EVENT_TYPE,
                 namespace: Text = DEFAULT_NAMESPACE,
                 sender: Text = None,
                 life: EventLife = EventLife.ONCE,
                 value_condition: ValueCondition = ValueCondition.EQUALS
                 ):
        """
        EventMeetConfig defines a set of criteria of the event we are interested in.

        :param event_key: The Key of the event(notification_service.base_notification.BaseEvent).
        :param event_value: The value of the event(notification_service.base_notification.BaseEvent).
        :param namespace: The namespace of the event(notification_service.base_notification.BaseEvent).
        :param event_type: (Optional) Type of the event(notification_service.base_notification.BaseEvent).
        :param sender: The sender of the event(notification_service.base_notification.BaseEvent).
        :param life: ai_flow.workflow.control_edge.EventLife
        :param value_condition: ai_flow.workflow.control_edge.MetValueCondition
        """
        self.event_type = event_type
        self.event_key = event_key
        self.event_value = event_value
        self.life = life
        self.value_condition = value_condition
        self.namespace = namespace
        self.sender = sender

    def __eq__(self, o: object) -> bool:
        if isinstance(o, EventMeetConfig):
            return self.__dict__ == o.__dict__
        else:
            return False

    def __repr__(self):
        return self.__dict__.__repr__()


class EventCondition(Jsonable):
    def __init__(self, events: List[EventMeetConfig], condition_type: ConditionType):
        """
        EventCondition combines a list of EventMeetConfig and a ConditionType to defines when the condition is met
        based on the events we see.

        :param events: A set of EventMeetConfig that the condition interested in.
        :param condition_type: condition_type defines when the condition is met.
        """
        self.events = events
        self.condition_type = condition_type

    def add_event(self,
                  event_key: Text,
                  event_value: Text,
                  event_type: Text = UNDEFINED_EVENT_TYPE,
                  namespace: Text = DEFAULT_NAMESPACE,
                  sender: Text = None,
                  life: EventLife = EventLife.ONCE,
                  value_condition: ValueCondition = ValueCondition.EQUALS):
        event_meet_config = EventMeetConfig(event_key, event_value, event_type,
                                            namespace, sender, life, value_condition)
        if event_meet_config in self.events:
            return self
        self.events.append(event_meet_config)
        return self

    def __eq__(self, o):
        if not isinstance(o, EventCondition):
            return False
        return self.events == o.events and self.condition_type == o.condition_type

    def __repr__(self):
        return self.__dict__.__str__()


class MeetAllEventCondition(EventCondition):
    """
    MeetAllEventCondition is met when all the events are met.
    """

    def __init__(self, events: List[EventMeetConfig] = None):
        if events is None:
            events = []
        super(MeetAllEventCondition, self).__init__(events, ConditionType.MEET_ALL)


class MeetAnyEventCondition(EventCondition):
    """
    MeetAnyEventCondition is met when any event is met.
    """

    def __init__(self, events: List[EventMeetConfig] = None):
        if events is None:
            events = []
        super(MeetAnyEventCondition, self).__init__(events, ConditionType.MEET_ANY)


class SchedulingRule(Jsonable):
    def __init__(self, event_condition: EventCondition, action: JobAction):
        """
        A SchedulingRule defines what TaskAction to take when the condition is met.
        :param event_condition: A config that defines the condition.
        :param action: The action to take when condition is met.
        """
        self.event_condition = event_condition
        self.action = action

    def __eq__(self, o):
        if not isinstance(o, SchedulingRule):
            return False
        return self.event_condition == o.event_condition and self.action == o.action


class ControlEdge(Edge):
    """
    ControlEdge defines event-based dependencies between jobs(ai_flow.workflow.job.Job).
    """

    def __init__(self,
                 destination: Text,
                 scheduling_rule: SchedulingRule,
                 ) -> None:
        """
        :param destination: The name of the job which depends on the scheduling rule
        :param scheduling_rule: The scheduling rule of the job
        """
        super().__init__('*', destination)
        self.scheduling_rule = scheduling_rule

    def __eq__(self, o):
        if not isinstance(o, ControlEdge):
            return False
        return self.destination == o.destination and self.source == o.source \
               and self.scheduling_rule == o.scheduling_rule


class AIFlowInternalEventType(object):
    """Per-defined some event types which are generated by ai flow system."""
    JOB_STATUS_CHANGED = "JOB_STATUS_CHANGED"  # Indicates the job(ai_flow.workflow.job.Job) status changed event.
    PERIODIC_ACTION = "PERIODIC_ACTION"  # Indicates the type of event that a job or workflow runs periodically.
    DATASET_CHANGED = "DATASET_CHANGED"  # Indicates the type of dataset changed event.



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""Helper function to generate a DAG and operators given some arguments."""

# [START subdag]
from airflow import DAG
from airflow.operators.dummy import DummyOperator
from airflow.utils.dates import days_ago


def subdag(parent_dag_name, child_dag_name, args):
    """
    Generate a DAG to be used as a subdag.

    :param str parent_dag_name: Id of the parent DAG
    :param str child_dag_name: Id of the child DAG
    :param dict args: Default arguments to provide to the subdag
    :return: DAG to use as a subdag
    :rtype: airflow.models.DAG
    """
    dag_subdag = DAG(
        dag_id=f'{parent_dag_name}.{child_dag_name}',
        default_args=args,
        start_date=days_ago(2),
        schedule_interval="@daily",
    )

    for i in range(5):
        DummyOperator(
            task_id='{}-task-{}'.format(child_dag_name, i + 1),
            default_args=args,
            dag=dag_subdag,
        )

    return dag_subdag


# [END subdag]



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Sync permission command"""
from airflow.models import DagBag
from airflow.utils import cli as cli_utils
from airflow.www.app import cached_app


@cli_utils.action_logging
def sync_perm(args):
    """Updates permissions for existing roles and DAGs"""
    appbuilder = cached_app().appbuilder  # pylint: disable=no-member
    print('Updating permission, view-menu for all existing roles')
    appbuilder.sm.sync_roles()
    # Add missing permissions for all the Base Views
    appbuilder.add_permissions(update_perms=True)
    print('Updating permission on all DAG views')
    dags = DagBag(read_dags_from_db=True).dags.values()
    for dag in dags:
        appbuilder.sm.sync_perm_for_dag(dag.dag_id, dag.access_control)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import os
import signal
import sys
import time
from subprocess import Popen
from tempfile import NamedTemporaryFile
from typing import Text, Any, List

from ai_flow.ai_graph.ai_graph import AISubGraph
from ai_flow.log import log_path_utils
from ai_flow.plugin_interface.job_plugin_interface import JobPluginFactory, JobHandle, JobRuntimeEnv, \
    JobController
from ai_flow.plugin_interface.scheduler_interface import JobExecutionInfo
from ai_flow.translator.translator import JobGenerator
from ai_flow.util import serialization_utils
from ai_flow.workflow.job import Job
from ai_flow.workflow.job_config import JobConfig
from ai_flow.workflow.status import Status
from ai_flow_plugins.job_plugins.python.python_job_config import PythonJobConfig
from ai_flow_plugins.job_plugins.utils.run_graph import RunGraph, build_run_graph


class PythonJob(Job):
    """
    PythonJob is the description of python type job.
    The run_graph_file field stores the serialized file of the RunGraph
    (ai_flow_plugins.job_plugins.python.python_job_plugin.RunGraph).
    """
    def __init__(self, job_config: JobConfig):
        super().__init__(job_config)
        self.run_graph_file: Text = None


class PythonJobHandle(JobHandle):

    def __init__(self, job: Job,
                 job_execution: JobExecutionInfo):
        super().__init__(job=job, job_execution=job_execution)
        """It saves the Popen object of the python child process."""
        self.sub_process = None


class PythonJobGenerator(JobGenerator):

    def generate(self, sub_graph: AISubGraph, resource_dir: Text = None) -> Job:
        python_job_config: PythonJobConfig = sub_graph.config
        run_graph: RunGraph = build_run_graph(sub_graph)
        job = PythonJob(job_config=python_job_config)
        with NamedTemporaryFile(mode='w+b', dir=resource_dir, prefix='{}_python_'.format(job.job_name), delete=False) as fp:
            job.run_graph_file = os.path.basename(fp.name)
            fp.write(serialization_utils.serialize(run_graph))
        return job


class PythonJobController(JobController):
    def submit_job(self, job: Job, job_runtime_env: JobRuntimeEnv = None) -> JobHandle:
        handle = PythonJobHandle(job=job, job_execution=job_runtime_env.job_execution_info)
        python_job: PythonJob = job
        run_graph_file = os.path.join(job_runtime_env.generated_dir, python_job.run_graph_file)

        env = os.environ.copy()
        if 'env' in job.job_config.properties:
            env.update(job.job_config.properties.get('env'))
        # Add PYTHONPATH
        copy_path = sys.path.copy()
        copy_path.insert(0, job_runtime_env.workflow_dir)
        copy_path.insert(0, job_runtime_env.python_dep_dir)
        env['PYTHONPATH'] = ':'.join(copy_path)

        current_path = os.path.dirname(__file__)
        script_path = os.path.join(current_path, 'python_run_main.py')
        if 'python_executable_path' in job.job_config.properties:
            python3_location = job.job_config.properties.get('python_executable_path')
        else:
            python3_location = sys.executable
        bash_command = [python3_location, script_path, run_graph_file, job_runtime_env.working_dir]

        stdout_log = log_path_utils.stdout_log_path(job_runtime_env.log_dir, job.job_name)
        stderr_log = log_path_utils.stderr_log_path(job_runtime_env.log_dir, job.job_name)
        if not os.path.exists(job_runtime_env.log_dir):
            os.makedirs(job_runtime_env.log_dir)

        sub_process = self.submit_python_process(bash_command=bash_command,
                                                 env=env,
                                                 working_dir=job_runtime_env.working_dir,
                                                 stdout_log=stdout_log,
                                                 stderr_log=stderr_log)
        handle.sub_process = sub_process
        return handle

    def stop_job(self, job_handle: JobHandle, job_runtime_env: Any = None):
        handle: PythonJobHandle = job_handle
        self.log.info('Output:')
        sub_process = handle.sub_process
        self.log.info('Sending SIGTERM signal to python process group')
        if sub_process and hasattr(sub_process, 'pid') and sub_process.poll() is None:
            while sub_process.poll() is None:
                try:
                    os.killpg(os.getpgid(sub_process.pid), signal.SIGTERM)
                except Exception as e:
                    self.log.error('Kill process {} failed! error {}'.format(sub_process.pid, str(e)))
                    time.sleep(1)

    def cleanup_job(self, job_handle: JobHandle, job_runtime_env: Any = None):
        pass

    def submit_python_process(self, bash_command: List, env, working_dir, stdout_log, stderr_log):

        def pre_exec():
            # Restore default signal disposition and invoke setsid
            for sig in ('SIGPIPE', 'SIGXFZ', 'SIGXFSZ'):
                if hasattr(signal, sig):
                    signal.signal(getattr(signal, sig), signal.SIG_DFL)
            os.setsid()

        self.log.info('Running command: %s', bash_command)
        with open(stdout_log, 'a') as out, open(stderr_log, 'a') as err:
            sub_process = Popen(  # pylint: disable=subprocess-popen-preexec-fn
                bash_command,
                stdout=out,
                stderr=err,
                cwd=working_dir,
                env=env,
                preexec_fn=pre_exec,
            )
        self.log.info('Process pid: %s', sub_process.pid)

        return sub_process

    def get_result(self, job_handle: JobHandle, blocking: bool = True) -> object:
        handle: PythonJobHandle = job_handle
        if blocking:
            handle.sub_process.wait()
            self.log.info('Command exited with return code %s', handle.sub_process.returncode)

            if handle.sub_process.returncode != 0:
                raise Exception('python returned a non-zero exit code {}.'.format(handle.sub_process.returncode))
            return None
        else:
            return None

    def get_job_status(self, job_handle: JobHandle) -> Status:
        handle: PythonJobHandle = job_handle
        sub_process = handle.sub_process
        if sub_process is not None:
            if sub_process.poll() is None:
                return Status.RUNNING
            else:
                if sub_process.returncode != 0:
                    return Status.FAILED
                else:
                    return Status.FINISHED
        else:
            return Status.INIT


class PythonJobPluginFactory(JobPluginFactory):
    def __init__(self) -> None:
        super().__init__()
        self._job_generator = PythonJobGenerator()
        self._job_controller = PythonJobController()

    def get_job_generator(self) -> JobGenerator:
        return self._job_generator

    def get_job_controller(self) -> JobController:
        return self._job_controller

    def job_type(self) -> Text:
        return "python"




#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from collections import Counter

from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
from airflow.utils.session import provide_session
from airflow.utils.state import State
from airflow.utils.trigger_rule import TriggerRule as TR


class TriggerRuleDep(BaseTIDep):
    """
    Determines if a task's upstream tasks are in a state that allows a given task instance
    to run.
    """

    NAME = "Trigger Rule"
    IGNOREABLE = True
    IS_TASK_DEP = True

    @staticmethod
    def _get_states_count_upstream_ti(ti, finished_tasks):
        """
        This function returns the states of the upstream tis for a specific ti in order to determine
        whether this ti can run in this iteration

        :param ti: the ti that we want to calculate deps for
        :type ti: airflow.models.TaskInstance
        :param finished_tasks: all the finished tasks of the dag_run
        :type finished_tasks: list[airflow.models.TaskInstance]
        """
        counter = Counter(task.state for task in finished_tasks if task.task_id in ti.task.upstream_task_ids)
        return (
            counter.get(State.SUCCESS, 0),
            counter.get(State.SKIPPED, 0),
            counter.get(State.FAILED, 0),
            counter.get(State.UPSTREAM_FAILED, 0),
            sum(counter.values()),
        )

    @provide_session
    def _get_dep_statuses(self, ti, session, dep_context):
        # Checking that all upstream dependencies have succeeded
        if not ti.task.upstream_list:
            yield self._passing_status(reason="The task instance did not have any upstream tasks.")
            return

        if ti.task.trigger_rule == TR.DUMMY:
            yield self._passing_status(reason="The task had a dummy trigger rule set.")
            return
        # see if the task name is in the task upstream for our task
        successes, skipped, failed, upstream_failed, done = self._get_states_count_upstream_ti(
            ti=ti, finished_tasks=dep_context.ensure_finished_tasks(ti.task.dag, ti.execution_date, session)
        )

        yield from self._evaluate_trigger_rule(
            ti=ti,
            successes=successes,
            skipped=skipped,
            failed=failed,
            upstream_failed=upstream_failed,
            done=done,
            flag_upstream_failed=dep_context.flag_upstream_failed,
            session=session,
        )

    @provide_session
    def _evaluate_trigger_rule(  # pylint: disable=too-many-branches
        self, ti, successes, skipped, failed, upstream_failed, done, flag_upstream_failed, session
    ):
        """
        Yields a dependency status that indicate whether the given task instance's trigger
        rule was met.

        :param ti: the task instance to evaluate the trigger rule of
        :type ti: airflow.models.TaskInstance
        :param successes: Number of successful upstream tasks
        :type successes: int
        :param skipped: Number of skipped upstream tasks
        :type skipped: int
        :param failed: Number of failed upstream tasks
        :type failed: int
        :param upstream_failed: Number of upstream_failed upstream tasks
        :type upstream_failed: int
        :param done: Number of completed upstream tasks
        :type done: int
        :param flag_upstream_failed: This is a hack to generate
            the upstream_failed state creation while checking to see
            whether the task instance is runnable. It was the shortest
            path to add the feature
        :type flag_upstream_failed: bool
        :param session: database session
        :type session: sqlalchemy.orm.session.Session
        """
        task = ti.task
        upstream = len(task.upstream_task_ids)
        trigger_rule = task.trigger_rule
        upstream_done = done >= upstream
        upstream_tasks_state = {
            "total": upstream,
            "successes": successes,
            "skipped": skipped,
            "failed": failed,
            "upstream_failed": upstream_failed,
            "done": done,
        }
        # TODO(aoen): Ideally each individual trigger rules would be its own class, but
        # this isn't very feasible at the moment since the database queries need to be
        # bundled together for efficiency.
        # handling instant state assignment based on trigger rules
        if flag_upstream_failed:
            if trigger_rule == TR.ALL_SUCCESS:
                if upstream_failed or failed:
                    ti.set_state(State.UPSTREAM_FAILED, session)
                elif skipped:
                    ti.set_state(State.SKIPPED, session)
            elif trigger_rule == TR.ALL_FAILED:
                if successes or skipped:
                    ti.set_state(State.SKIPPED, session)
            elif trigger_rule == TR.ONE_SUCCESS:
                if upstream_done and not successes:
                    ti.set_state(State.SKIPPED, session)
            elif trigger_rule == TR.ONE_FAILED:
                if upstream_done and not (failed or upstream_failed):
                    ti.set_state(State.SKIPPED, session)
            elif trigger_rule == TR.NONE_FAILED:
                if upstream_failed or failed:
                    ti.set_state(State.UPSTREAM_FAILED, session)
            elif trigger_rule == TR.NONE_FAILED_OR_SKIPPED:
                if upstream_failed or failed:
                    ti.set_state(State.UPSTREAM_FAILED, session)
                elif skipped == upstream:
                    ti.set_state(State.SKIPPED, session)
            elif trigger_rule == TR.NONE_SKIPPED:
                if skipped:
                    ti.set_state(State.SKIPPED, session)

        if trigger_rule == TR.ONE_SUCCESS:
            if successes <= 0:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires one upstream "
                    "task success, but none were found. "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.ONE_FAILED:
            if not failed and not upstream_failed:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires one upstream "
                    "task failure, but none were found. "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.ALL_SUCCESS:
            num_failures = upstream - successes
            if num_failures > 0:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires all upstream "
                    "tasks to have succeeded, but found {} non-success(es). "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, num_failures, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.ALL_FAILED:
            num_successes = upstream - failed - upstream_failed
            if num_successes > 0:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires all upstream "
                    "tasks to have failed, but found {} non-failure(s). "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, num_successes, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.ALL_DONE:
            if not upstream_done:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires all upstream "
                    "tasks to have completed, but found {} task(s) that "
                    "were not done. upstream_tasks_state={}, "
                    "upstream_task_ids={}".format(
                        trigger_rule, upstream_done, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.NONE_FAILED:
            num_failures = upstream - successes - skipped
            if num_failures > 0:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires all upstream "
                    "tasks to have succeeded or been skipped, but found {} non-success(es). "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, num_failures, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.NONE_FAILED_OR_SKIPPED:
            num_failures = upstream - successes - skipped
            if num_failures > 0:
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires all upstream "
                    "tasks to have succeeded or been skipped, but found {} non-success(es). "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, num_failures, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        elif trigger_rule == TR.NONE_SKIPPED:
            if not upstream_done or (skipped > 0):
                yield self._failing_status(
                    reason="Task's trigger rule '{}' requires all upstream "
                    "tasks to not have been skipped, but found {} task(s) skipped. "
                    "upstream_tasks_state={}, upstream_task_ids={}".format(
                        trigger_rule, skipped, upstream_tasks_state, task.upstream_task_ids
                    )
                )
        else:
            yield self._failing_status(reason=f"No strategy to evaluate trigger rule '{trigger_rule}'.")



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import unittest
from typing import Text, List, Dict

from ai_flow.context.project_context import ProjectContext
from ai_flow.scheduler.scheduler_factory import SchedulerFactory
from ai_flow.plugin_interface.scheduler_interface import Scheduler
from ai_flow.workflow.workflow import Workflow
from ai_flow.plugin_interface.scheduler_interface import JobExecutionInfo, WorkflowExecutionInfo, WorkflowInfo


class UnitTestScheduler(Scheduler):
    def delete_workflow(self, project_name: Text, workflow_name: Text) -> WorkflowInfo:
        pass

    def start_job_execution(self, job_name: Text, workflow_execution_id: Text) -> JobExecutionInfo:
        pass

    def stop_job_execution(self, job_name: Text, workflow_execution_id: Text) -> JobExecutionInfo:
        pass

    def restart_job_execution(self, job_name: Text, workflow_execution_id: Text) -> JobExecutionInfo:
        pass

    def get_job_executions(self, job_name: Text, workflow_execution_id: Text) -> List[JobExecutionInfo]:
        pass

    def list_job_executions(self, workflow_execution_id: Text) -> List[JobExecutionInfo]:
        pass

    def submit_workflow(self, workflow: Workflow, project_context: ProjectContext) -> WorkflowInfo:
        pass

    def pause_workflow_scheduling(self, project_name: Text, workflow_name: Text) -> WorkflowInfo:
        pass

    def resume_workflow_scheduling(self, project_name: Text, workflow_name: Text) -> WorkflowInfo:
        pass

    def start_new_workflow_execution(self, project_name: Text, workflow_name: Text) -> WorkflowExecutionInfo:
        pass

    def stop_all_workflow_execution(self, project_name: Text, workflow_name: Text) -> List[WorkflowExecutionInfo]:
        pass

    def stop_workflow_execution(self, workflow_execution_id: Text) -> WorkflowExecutionInfo:
        pass

    def get_workflow_execution(self, workflow_execution_id: Text) -> WorkflowExecutionInfo:
        pass

    def list_workflow_executions(self, project_name: Text, workflow_name: Text) -> List[WorkflowExecutionInfo]:
        pass


class TestSchedulerFactory(unittest.TestCase):

    def test_create_scheduler(self):
        print(Scheduler.__class__.__name__, Scheduler.__class__.__module__)
        class_name = 'ai_flow.test.scheduler.test_scheduler_factory.UnitTestScheduler'
        sc = SchedulerFactory.create_scheduler(class_name=class_name, config={})
        self.assertTrue(isinstance(sc, Scheduler))



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
from ai_flow.model_center.entity.model_version import ModelVersion
from ai_flow.model_center.entity.model_version_stage import ModelVersionStage
from ai_flow.model_center.entity.model_version_status import ModelVersionStatus
from ai_flow.protobuf.message_pb2 import ModelVersionMeta
from ai_flow.endpoint.server import stringValue


class ModelVersionDetail(ModelVersion):
    """
    AIFlow entity for Model Version Detailed.
    Provides additional metadata data for model version in addition to information in
    :py:class:`ai_flow.model_center.entity.ModelVersion`.
    """

    def __init__(self, model_name, model_version, model_path=None,
                 model_type=None, version_desc=None,
                 version_status=None, current_stage=None):
        # Constructor is called only from within the system by various backend stores.
        super(ModelVersionDetail, self).__init__(model_name=model_name,
                                                 model_version=model_version)
        self._model_path = model_path
        self._model_type = model_type
        self._version_desc = version_desc
        self._version_status = version_status
        self._current_stage = current_stage

    @property
    def model_path(self):
        """String. Model source path for the model."""
        return self._model_path

    @property
    def model_type(self):
        """String. Model type of the model."""
        return self._model_type

    @property
    def version_desc(self):
        """String. Description of the model version."""
        return self._version_desc

    @property
    def version_status(self):
        """String. Current Model Registry status for this model."""
        return self._version_status

    @property
    def current_stage(self):
        """String. Current stage of this model version."""
        return self._current_stage

    @classmethod
    def _properties(cls):
        # aggregate with base class properties since cls.__dict__ does not do it automatically
        return sorted(cls._get_properties_helper() + ModelVersion._properties())

    # proto mappers
    @classmethod
    def from_proto(cls, proto):
        if proto is None:
            return None
        else:
            return cls(proto.model_name, proto.model_version,
                       proto.model_path.value if proto.HasField("model_path") else None,
                       proto.model_type.value if proto.HasField("model_type") else None,
                       proto.version_desc.value if proto.HasField("version_desc") else None,
                       proto.version_status, proto.current_stage)

    def to_meta_proto(self):
        return ModelVersionMeta(model_name=self.model_name, model_version=self.model_version,
                                model_path=stringValue(self.model_path),
                                model_type=stringValue(self.model_type),
                                version_desc=stringValue(self.version_desc),
                                version_status=ModelVersionStatus.from_string(self.version_status),
                                current_stage=ModelVersionStage.from_string(self.current_stage.upper()))



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import pickle

_BYTES_ENCODE = "ISO-8859-1"


def serialize(o: object) -> bytes:
    return pickle.dumps(o)


def deserialize(s: bytes) -> object:
    return pickle.loads(s)


def read_object_from_serialized_file(file_path):
    with open(file_path, 'rb') as f:
        serialized_bytes = f.read()
    return deserialize(serialized_bytes)


# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""Config sub-commands"""
import getpass
import locale
import logging
import os
import platform
import subprocess
import sys
from typing import Optional
from urllib.parse import urlsplit, urlunsplit

import requests
import tenacity
from rich.console import Console

from airflow import configuration
from airflow.cli.simple_table import SimpleTable
from airflow.providers_manager import ProvidersManager
from airflow.typing_compat import Protocol
from airflow.utils.cli import suppress_logs_and_warning
from airflow.version import version as airflow_version

log = logging.getLogger(__name__)


class Anonymizer(Protocol):
    """Anonymizer protocol."""

    def process_path(self, value) -> str:
        """Remove pii from paths"""

    def process_username(self, value) -> str:
        """Remove pii from username"""

    def process_url(self, value) -> str:
        """Remove pii from URL"""


class NullAnonymizer(Anonymizer):
    """Do nothing."""

    def _identity(self, value):
        return value

    process_path = process_username = process_url = _identity

    del _identity


class PiiAnonymizer(Anonymizer):
    """Remove personally identifiable info from path."""

    def __init__(self):
        home_path = os.path.expanduser("~")
        username = getpass.getuser()
        self._path_replacements = {home_path: "${HOME}", username: "${USER}"}

    def process_path(self, value):
        if not value:
            return value
        for src, target in self._path_replacements.items():
            value = value.replace(src, target)
        return value

    def process_username(self, value):
        if not value:
            return value
        return value[0] + "..." + value[-1]

    def process_url(self, value):
        if not value:
            return value

        url_parts = urlsplit(value)
        netloc = None
        if url_parts.netloc:
            # unpack
            userinfo = None
            host = None
            username = None
            password = None

            if "@" in url_parts.netloc:
                userinfo, _, host = url_parts.netloc.partition("@")
            else:
                host = url_parts.netloc
            if userinfo:
                if ":" in userinfo:
                    username, _, password = userinfo.partition(":")
                else:
                    username = userinfo

            # anonymize
            username = self.process_username(username) if username else None
            password = "PASSWORD" if password else None

            # pack
            if username and password and host:
                netloc = username + ":" + password + "@" + host
            elif username and host:
                netloc = username + "@" + host
            elif password and host:
                netloc = ":" + password + "@" + host
            elif host:
                netloc = host
            else:
                netloc = ""

        return urlunsplit((url_parts.scheme, netloc, url_parts.path, url_parts.query, url_parts.fragment))


class OperatingSystem:
    """Operating system"""

    WINDOWS = "Windows"
    LINUX = "Linux"
    MACOSX = "Mac OS"
    CYGWIN = "Cygwin"

    @staticmethod
    def get_current() -> Optional[str]:
        """Get current operating system"""
        if os.name == "nt":
            return OperatingSystem.WINDOWS
        elif "linux" in sys.platform:
            return OperatingSystem.LINUX
        elif "darwin" in sys.platform:
            return OperatingSystem.MACOSX
        elif "cygwin" in sys.platform:
            return OperatingSystem.CYGWIN
        return None


class Architecture:
    """Compute architecture"""

    X86_64 = "x86_64"
    X86 = "x86"
    PPC = "ppc"
    ARM = "arm"

    @staticmethod
    def get_current():
        """Get architecture"""
        return _MACHINE_TO_ARCHITECTURE.get(platform.machine().lower())


_MACHINE_TO_ARCHITECTURE = {
    "amd64": Architecture.X86_64,
    "x86_64": Architecture.X86_64,
    "i686-64": Architecture.X86_64,
    "i386": Architecture.X86,
    "i686": Architecture.X86,
    "x86": Architecture.X86,
    "ia64": Architecture.X86,  # Itanium is different x64 arch, treat it as the common x86.
    "powerpc": Architecture.PPC,
    "power macintosh": Architecture.PPC,
    "ppc64": Architecture.PPC,
    "armv6": Architecture.ARM,
    "armv6l": Architecture.ARM,
    "arm64": Architecture.ARM,
    "armv7": Architecture.ARM,
    "armv7l": Architecture.ARM,
}


class _BaseInfo:
    def info(self, console: Console) -> None:
        """
        Print required information to provided console.
        You should implement this function in custom classes.
        """
        raise NotImplementedError()

    def show(self) -> None:
        """Shows info"""
        console = Console()
        self.info(console)

    def render_text(self) -> str:
        """Exports the info to string"""
        console = Console(record=True)
        with console.capture():
            self.info(console)
        return console.export_text()


class AirflowInfo(_BaseInfo):
    """All information related to Airflow, system and other."""

    def __init__(self, anonymizer: Anonymizer):
        self.airflow_version = airflow_version
        self.system = SystemInfo(anonymizer)
        self.tools = ToolsInfo(anonymizer)
        self.paths = PathsInfo(anonymizer)
        self.config = ConfigInfo(anonymizer)
        self.provider = ProvidersInfo()

    def info(self, console: Console):
        console.print(
            f"[bold][green]Apache Airflow[/bold][/green]: {self.airflow_version}\n", highlight=False
        )
        self.system.info(console)
        self.tools.info(console)
        self.paths.info(console)
        self.config.info(console)
        self.provider.info(console)


class SystemInfo(_BaseInfo):
    """Basic system and python information"""

    def __init__(self, anonymizer: Anonymizer):
        self.operating_system = OperatingSystem.get_current()
        self.arch = Architecture.get_current()
        self.uname = platform.uname()
        self.locale = locale.getdefaultlocale()
        self.python_location = anonymizer.process_path(sys.executable)
        self.python_version = sys.version.replace("\n", " ")

    def info(self, console: Console):
        table = SimpleTable(title="System info")
        table.add_column()
        table.add_column(width=100)
        table.add_row("OS", self.operating_system or "NOT AVAILABLE")
        table.add_row("architecture", self.arch or "NOT AVAILABLE")
        table.add_row("uname", str(self.uname))
        table.add_row("locale", str(self.locale))
        table.add_row("python_version", self.python_version)
        table.add_row("python_location", self.python_location)
        console.print(table)


class PathsInfo(_BaseInfo):
    """Path information"""

    def __init__(self, anonymizer: Anonymizer):
        system_path = os.environ.get("PATH", "").split(os.pathsep)

        self.airflow_home = anonymizer.process_path(configuration.get_airflow_home())
        self.system_path = [anonymizer.process_path(p) for p in system_path]
        self.python_path = [anonymizer.process_path(p) for p in sys.path]
        self.airflow_on_path = any(
            os.path.exists(os.path.join(path_elem, "airflow")) for path_elem in system_path
        )

    def info(self, console: Console):
        table = SimpleTable(title="Paths info")
        table.add_column()
        table.add_column(width=150)
        table.add_row("airflow_home", self.airflow_home)
        table.add_row("system_path", os.pathsep.join(self.system_path))
        table.add_row("python_path", os.pathsep.join(self.python_path))
        table.add_row("airflow_on_path", str(self.airflow_on_path))
        console.print(table)


class ProvidersInfo(_BaseInfo):
    """providers information"""

    def info(self, console: Console):
        table = SimpleTable(title="Providers info")
        table.add_column()
        table.add_column(width=150)
        for _, provider in ProvidersManager().providers.values():
            table.add_row(provider['package-name'], provider['versions'][0])
        console.print(table)


class ConfigInfo(_BaseInfo):
    """Most critical config properties"""

    def __init__(self, anonymizer: Anonymizer):
        self.executor = configuration.conf.get("core", "executor")
        self.sql_alchemy_conn = anonymizer.process_url(
            configuration.conf.get("core", "SQL_ALCHEMY_CONN", fallback="NOT AVAILABLE")
        )
        self.dags_folder = anonymizer.process_path(
            configuration.conf.get("core", "dags_folder", fallback="NOT AVAILABLE")
        )
        self.plugins_folder = anonymizer.process_path(
            configuration.conf.get("core", "plugins_folder", fallback="NOT AVAILABLE")
        )
        self.base_log_folder = anonymizer.process_path(
            configuration.conf.get("logging", "base_log_folder", fallback="NOT AVAILABLE")
        )
        self.remote_base_log_folder = anonymizer.process_path(
            configuration.conf.get("logging", "remote_base_log_folder", fallback="NOT AVAILABLE")
        )

    @property
    def task_logging_handler(self):
        """Returns task logging handler."""

        def get_fullname(o):
            module = o.__class__.__module__
            if module is None or module == str.__class__.__module__:
                return o.__class__.__name__  # Avoid reporting __builtin__
            else:
                return module + '.' + o.__class__.__name__

        try:
            handler_names = [get_fullname(handler) for handler in logging.getLogger('airflow.task').handlers]
            return ", ".join(handler_names)
        except Exception:  # noqa pylint: disable=broad-except
            return "NOT AVAILABLE"

    def info(self, console: Console):
        table = SimpleTable(title="Config info")
        table.add_column()
        table.add_column(width=150)
        table.add_row("executor", self.executor)
        table.add_row("task_logging_handler", self.task_logging_handler)
        table.add_row("sql_alchemy_conn", self.sql_alchemy_conn)
        table.add_row("dags_folder", self.dags_folder)
        table.add_row("plugins_folder", self.plugins_folder)
        table.add_row("base_log_folder", self.base_log_folder)
        console.print(table)


class ToolsInfo(_BaseInfo):
    """The versions of the tools that Airflow uses"""

    def __init__(self, anonymize: Anonymizer):
        del anonymize  # Nothing to anonymize here.
        self.git_version = self._get_version(["git", "--version"])
        self.ssh_version = self._get_version(["ssh", "-V"])
        self.kubectl_version = self._get_version(["kubectl", "version", "--short=True", "--client=True"])
        self.gcloud_version = self._get_version(["gcloud", "version"], grep=b"Google Cloud SDK")
        self.cloud_sql_proxy_version = self._get_version(["cloud_sql_proxy", "--version"])
        self.mysql_version = self._get_version(["mysql", "--version"])
        self.sqlite3_version = self._get_version(["sqlite3", "--version"])
        self.psql_version = self._get_version(["psql", "--version"])

    def _get_version(self, cmd, grep=None):
        """Return tools version."""
        try:
            proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        except OSError:
            return "NOT AVAILABLE"
        stdoutdata, _ = proc.communicate()
        data = [f for f in stdoutdata.split(b"\n") if f]
        if grep:
            data = [line for line in data if grep in line]
        if len(data) != 1:
            return "NOT AVAILABLE"
        else:
            return data[0].decode()

    def info(self, console: Console):
        table = SimpleTable(title="Tools info")
        table.add_column()
        table.add_column(width=150)
        table.add_row("git", self.git_version)
        table.add_row("ssh", self.ssh_version)
        table.add_row("kubectl", self.kubectl_version)
        table.add_row("gcloud", self.gcloud_version)
        table.add_row("cloud_sql_proxy", self.cloud_sql_proxy_version)
        table.add_row("mysql", self.mysql_version)
        table.add_row("sqlite3", self.sqlite3_version)
        table.add_row("psql", self.psql_version)
        console.print(table)


class FileIoException(Exception):
    """Raises when error happens in FileIo.io integration"""


@tenacity.retry(
    stop=tenacity.stop_after_attempt(5),
    wait=tenacity.wait_exponential(multiplier=1, max=10),
    retry=tenacity.retry_if_exception_type(FileIoException),
    before=tenacity.before_log(log, logging.DEBUG),
    after=tenacity.after_log(log, logging.DEBUG),
)
def _upload_text_to_fileio(content):
    """Upload text file to File.io service and return lnk"""
    resp = requests.post("https://file.io", data={"text": content})
    if not resp.ok:
        print(resp.json())
        raise FileIoException("Failed to send report to file.io service.")
    try:
        return resp.json()["link"]
    except ValueError as e:
        log.debug(e)
        raise FileIoException("Failed to send report to file.io service.")


def _send_report_to_fileio(info):
    print("Uploading report to file.io service.")
    try:
        link = _upload_text_to_fileio(str(info))
        print("Report uploaded.")
        print(link)
        print()
    except FileIoException as ex:
        print(str(ex))


@suppress_logs_and_warning()
def show_info(args):
    """Show information related to Airflow, system and other."""
    # Enforce anonymization, when file_io upload is tuned on.
    anonymizer = PiiAnonymizer() if args.anonymize or args.file_io else NullAnonymizer()
    info = AirflowInfo(anonymizer)
    if args.file_io:
        _send_report_to_fileio(info.render_text())
    else:
        info.show()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""index-faskfail

Revision ID: 9635ae0956e7
Revises: 856955da8476
Create Date: 2018-06-17 21:40:01.963540

"""
from alembic import op

# revision identifiers, used by Alembic.
revision = '9635ae0956e7'
down_revision = '856955da8476'
branch_labels = None
depends_on = None


def upgrade():  # noqa: D103
    op.create_index(
        'idx_task_fail_dag_task_date', 'task_fail', ['dag_id', 'task_id', 'execution_date'], unique=False
    )


def downgrade():  # noqa: D103
    op.drop_index('idx_task_fail_dag_task_date', table_name='task_fail')



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
from abc import ABC, abstractmethod
from typing import Text, Dict

from ai_flow.common.configuration import AIFlowConfiguration
from ai_flow.common.module_load import import_string


class BlobConfig(AIFlowConfiguration):

    def __init__(self, config: Dict):
        super().__init__()
        self['blob_manager_class'] = None
        self['blob_manager_config'] = {}
        if config is None or 'blob_manager_class' not in config:
            raise Exception(
                'The `blob_manager_class` option of blob config is not configured. '
                'Please add the `blob_manager_class` option under the `blob` option!')
        self.set_blob_manager_class(config.get('blob_manager_class'))
        if config.get('blob_manager_config') is not None:
            self['blob_manager_config'].update(config.get('blob_manager_config'))

    def blob_manager_class(self):
        return self.get('blob_manager_class')

    def set_blob_manager_class(self, value):
        self['blob_manager_class'] = value

    def blob_manager_config(self):
        if 'blob_manager_config' not in self:
            return None
        return self['blob_manager_config']

    def set_blob_manager_config(self, value):
        self['blob_manager_config'] = value


class BlobManager(ABC):
    """
    A BlobManager is responsible for uploading and downloading files and resource for an execution of an ai flow project.
    """
    def __init__(self, config: Dict):
        self.config = config

    @abstractmethod
    def upload_project(self, workflow_snapshot_id: Text, project_path: Text) -> Text:
        """
        upload a given project to blob server for remote execution.

        :param workflow_snapshot_id: It is the unique identifier for each workflow generation.
        :param project_path: the path of this project.
        :return the uri of the uploaded project file in blob server.
        """
        pass

    @abstractmethod
    def download_project(self, workflow_snapshot_id, remote_path: Text, local_path: Text = None) -> Text:
        """
        download the needed resource from remote blob server to local process for remote execution.

        :param workflow_snapshot_id: It is the unique identifier for each workflow generation.
        :param remote_path: The project package uri.
        :param local_path: Download file root path.
        :return Local project path.
        """
        pass

    def cleanup_project(self, workflow_snapshot_id, remote_path: Text):
        """
        clean up the project files downloaded or created during this execution.
        :param workflow_snapshot_id: It is the unique identifier for each workflow generation.
        :param remote_path: The project package uri.
        """
        pass


class BlobManagerFactory:
    @classmethod
    def create_blob_manager(cls, class_name, config: Dict) -> BlobManager:
        """
        :param class_name: The class name of a (~class:`ai_flow.plugin_interface.blob_manager_interface.BlobManager`)
        :param config: The configuration of the BlobManager.
        """
        if class_name is None:
            return None
        class_object = import_string(class_name)
        return class_object(config)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

from ai_flow.common.status import Status
from ai_flow.meta.dataset_meta import DataType
from ai_flow.metadata_store.utils.MetaToProto import MetaToProto
from ai_flow.metadata_store.utils.ProtoToMeta import ProtoToMeta
from ai_flow.protobuf import metadata_service_pb2_grpc
from ai_flow.protobuf.message_pb2 import DataTypeProto
from ai_flow.endpoint.client.model_center_client import ModelCenterClient
from ai_flow.endpoint.server.util import _wrap_meta_response, transform_dataset_meta, \
    _warp_dataset_list_response, _wrap_delete_response, transform_model_relation_meta, \
    _warp_model_relation_list_response, _warp_model_version_relation_list_response, \
    transform_model_version_relation_meta, _warp_project_list_response, transform_project_meta, catch_exception, \
    transform_model_meta, transform_model_version_meta, transform_artifact_meta, _warp_artifact_list_response, \
    transform_workflow_meta, _wrap_workflow_list_response
from ai_flow.store.sqlalchemy_store import SqlAlchemyStore
from ai_flow.store.mongo_store import MongoStore
from ai_flow.store.db.db_util import extract_db_engine_from_uri, parse_mongo_uri
from ai_flow.endpoint.server.server_config import DBType


class MetadataService(metadata_service_pb2_grpc.MetadataServiceServicer):
    def __init__(self, db_uri, server_uri):
        db_engine = extract_db_engine_from_uri(db_uri)
        if DBType.value_of(db_engine) == DBType.MONGODB:
            username, password, host, port, db = parse_mongo_uri(db_uri)
            self.store = MongoStore(host=host,
                                    port=int(port),
                                    username=username,
                                    password=password,
                                    db=db)
        else:
            self.store = SqlAlchemyStore(db_uri)
        self.model_center_client = ModelCenterClient(server_uri)

    '''dataset api'''

    @catch_exception
    def getDatasetById(self, request, context):
        dataset = self.store.get_dataset_by_id(request.id)
        return _wrap_meta_response(MetaToProto.dataset_meta_to_proto(dataset))

    @catch_exception
    def getDatasetByName(self, request, context):
        dataset = self.store.get_dataset_by_name(request.name)
        return _wrap_meta_response(MetaToProto.dataset_meta_to_proto(dataset))

    @catch_exception
    def registerDataset(self, request, context):
        dataset = transform_dataset_meta(request.dataset)
        dataset_meta = self.store.register_dataset(name=dataset.name,
                                                   data_format=dataset.data_format,
                                                   description=dataset.description,
                                                   uri=dataset.uri,
                                                   properties=dataset.properties,
                                                   name_list=dataset.schema.name_list,
                                                   type_list=dataset.schema.type_list)
        return _wrap_meta_response(MetaToProto.dataset_meta_to_proto(dataset_meta))

    @catch_exception
    def registerDatasetWithCatalog(self, request, context):
        dataset = transform_dataset_meta(request.dataset)
        dataset_meta = self.store.register_dataset_with_catalog(name=dataset.name,
                                                                catalog_name=dataset.catalog_name,
                                                                catalog_type=dataset.catalog_type,
                                                                catalog_database=dataset.catalog_database,
                                                                catalog_connection_uri=dataset.catalog_connection_uri,
                                                                catalog_table=dataset.catalog_table)
        return _wrap_meta_response(MetaToProto.dataset_meta_to_proto(dataset_meta))

    @catch_exception
    def registerDatasets(self, request, context):
        _datasets = ProtoToMeta.proto_to_dataset_meta_list(request.datasets)
        response = self.store.register_datasets(_datasets)
        return _warp_dataset_list_response(response)

    @catch_exception
    def updateDataset(self, request, context):
        properties = None if request.properties == {} else request.properties
        name_list = request.name_list
        type_list = request.type_list
        if not name_list:
            name_list = None
        if not type_list:
            data_type_list = None
        else:
            data_type_list = []
            for data_type in type_list:
                data_type_list.append(DataType(DataTypeProto.Name(data_type)))
        dataset_meta = self.store.update_dataset(dataset_name=request.name,
                                                 data_format=request.data_format.value if request.HasField(
                                                     'data_format') else None,
                                                 description=request.description.value if request.HasField(
                                                     'description') else None,
                                                 uri=request.uri.value if request.HasField('uri') else None,
                                                 properties=properties,
                                                 name_list=name_list,
                                                 type_list=data_type_list,
                                                 catalog_name=request.catalog_name.value if request.HasField(
                                                     'catalog_name') else None,
                                                 catalog_type=request.catalog_type.value if request.HasField(
                                                     'catalog_type') else None,
                                                 catalog_database=request.catalog_database.value if request.HasField(
                                                     'catalog_database') else None,
                                                 catalog_connection_uri=request.catalog_connection_uri.value \
                                                     if request.HasField('catalog_connection_uri') else None,
                                                 catalog_table=request.catalog_table.value if request.HasField(
                                                     'catalog_table') else None)
        return _wrap_meta_response(MetaToProto.dataset_meta_to_proto(dataset_meta))

    @catch_exception
    def listDatasets(self, request, context):
        dataset_meta_list = self.store.list_datasets(request.page_size, request.offset)
        return _warp_dataset_list_response(dataset_meta_list)

    @catch_exception
    def deleteDatasetById(self, request, context):
        status = self.store.delete_dataset_by_id(request.id)
        return _wrap_delete_response(status)

    @catch_exception
    def deleteDatasetByName(self, request, context):
        status = self.store.delete_dataset_by_name(request.name)
        return _wrap_delete_response(status)

    '''model relation api'''

    @catch_exception
    def getModelRelationById(self, request, context):
        model_meta = self.store.get_model_relation_by_id(request.id)
        return _wrap_meta_response(MetaToProto.model_relation_meta_to_proto(model_meta))

    @catch_exception
    def getModelRelationByName(self, request, context):
        model_meta = self.store.get_model_relation_by_name(request.name)
        return _wrap_meta_response(MetaToProto.model_relation_meta_to_proto(model_meta))

    @catch_exception
    def registerModelRelation(self, request, context):
        model = transform_model_relation_meta(request.model_relation)
        response = self.store.register_model_relation(name=model.name, project_id=model.project_id)
        return _wrap_meta_response(MetaToProto.model_relation_meta_to_proto(response))

    @catch_exception
    def listModelRelation(self, request, context):
        model_list = self.store.list_model_relation(request.page_size, request.offset)
        return _warp_model_relation_list_response(model_list)

    @catch_exception
    def deleteModelRelationById(self, request, context):
        status = self.store.delete_model_relation_by_id(request.id)
        return _wrap_delete_response(status)

    @catch_exception
    def deleteModelRelationByName(self, request, context):
        status = self.store.delete_model_relation_by_name(request.name)
        return _wrap_delete_response(status)

    '''model api'''

    @catch_exception
    def getModelById(self, request, context):
        model_relation = self.store.get_model_relation_by_id(request.id)
        if model_relation is None:
            model_detail = None
        else:
            model_detail = self.model_center_client.get_registered_model_detail(model_relation.name)
        return _wrap_meta_response(MetaToProto.model_meta_to_proto(model_relation, model_detail))

    @catch_exception
    def getModelByName(self, request, context):
        model_relation = self.store.get_model_relation_by_name(request.name)
        model_detail = self.model_center_client.get_registered_model_detail(request.name)
        return _wrap_meta_response(MetaToProto.model_meta_to_proto(model_relation, model_detail))

    @catch_exception
    def registerModel(self, request, context):
        model = transform_model_meta(request.model)
        model_detail = self.model_center_client.create_registered_model(model.name,
                                                                        model.model_desc)
        model_relation = self.store.register_model_relation(name=model.name, project_id=model.project_id)
        return _wrap_meta_response(MetaToProto.model_meta_to_proto(model_relation, model_detail))

    @catch_exception
    def deleteModelById(self, request, context):
        model_relation = self.store.get_model_relation_by_id(request.id)
        if model_relation is None:
            return _wrap_delete_response(Status.ERROR)
        else:
            model_relation_status = self.store.delete_model_relation_by_id(request.id)
            self.model_center_client.delete_registered_model(model_relation.name)
            return _wrap_delete_response(model_relation_status)

    @catch_exception
    def deleteModelByName(self, request, context):
        model_relation_status = self.store.delete_model_relation_by_name(request.name)
        self.model_center_client.delete_registered_model(request.name)
        return _wrap_delete_response(model_relation_status)

    '''model version relation api'''

    @catch_exception
    def getModelVersionRelationByVersion(self, request, context):
        model_version_meta = self.store.get_model_version_relation_by_version(request.name, request.model_id)
        return _wrap_meta_response(MetaToProto.model_version_relation_meta_to_proto(model_version_meta))

    @catch_exception
    def listModelVersionRelation(self, request, context):
        model_version_meta_list = self.store.list_model_version_relation(request.model_id, request.page_size,
                                                                         request.offset)
        return _warp_model_version_relation_list_response(model_version_meta_list)

    @catch_exception
    def registerModelVersionRelation(self, request, context):
        model_version = transform_model_version_relation_meta(request.model_version_relation)
        response = self.store.register_model_version_relation(version=model_version.version,
                                                              model_id=model_version.model_id,
                                                              project_snapshot_id=model_version.project_snapshot_id)
        return _wrap_meta_response(MetaToProto.model_version_relation_meta_to_proto(response))

    @catch_exception
    def deleteModelVersionRelationByVersion(self, request, context):
        status = self.store.delete_model_version_relation_by_version(request.name, request.model_id)
        return _wrap_delete_response(status)

    '''model version api'''

    @catch_exception
    def getModelVersionByVersion(self, request, context):
        model_version_relation = self.store.get_model_version_relation_by_version(request.name, request.model_id)
        if model_version_relation is None:
            model_version_detail = None
        else:
            model_relation = self.store.get_model_relation_by_id(model_version_relation.model_id)
            model_version_detail = self.model_center_client.get_model_version_detail(model_relation.name, request.name)
        return _wrap_meta_response(
            MetaToProto.model_version_meta_to_proto(model_version_relation, model_version_detail))

    @catch_exception
    def registerModelVersion(self, request, context):
        model_version = transform_model_version_meta(request.model_version)
        model_relation = self.store.get_model_relation_by_id(model_version.model_id)
        model_version_detail = self.model_center_client.create_model_version(model_relation.name,
                                                                             model_version.model_path,
                                                                             model_version.model_type,
                                                                             model_version.version_desc,
                                                                             request.model_version.current_stage)
        model_version_relation = self.store.register_model_version_relation(version=model_version_detail.model_version,
                                                                            model_id=model_version.model_id,
                                                                            project_snapshot_id=
                                                                            model_version.project_snapshot_id)
        return _wrap_meta_response(
            MetaToProto.model_version_meta_to_proto(model_version_relation, model_version_detail))

    @catch_exception
    def deleteModelVersionByVersion(self, request, context):
        model_version_relation = self.store.get_model_version_relation_by_version(request.name, request.model_id)
        if model_version_relation is None:
            return _wrap_delete_response(Status.ERROR)
        else:
            model_version__status = self.store.delete_model_version_relation_by_version(request.name, request.model_id)
            model_relation = self.store.get_model_relation_by_id(model_version_relation.model_id)
            if model_relation is not None:
                self.model_center_client.delete_model_version(model_relation.name, request.name)
            return _wrap_delete_response(model_version__status)

    @catch_exception
    def getDeployedModelVersion(self, request, context):
        model_version_detail = self.store.get_deployed_model_version(request.name)
        if model_version_detail is None:
            model_version_relation = None
        else:
            model_relation = self.store.get_model_relation_by_name(request.name)
            model_version_relation = self.store.get_model_version_relation_by_version(
                model_version_detail.model_version,
                model_relation.uuid)
        return _wrap_meta_response(
            MetaToProto.model_version_store_to_proto(model_version_relation, model_version_detail))

    @catch_exception
    def getLatestValidatedModelVersion(self, request, context):
        model_version_detail = self.store.get_latest_validated_model_version(request.name)
        if model_version_detail is None:
            model_version_relation = None
        else:
            model_relation = self.store.get_model_relation_by_name(request.name)
            model_version_relation = self.store.get_model_version_relation_by_version(
                model_version_detail.model_version,
                model_relation.uuid)
        return _wrap_meta_response(
            MetaToProto.model_version_store_to_proto(model_version_relation, model_version_detail))

    @catch_exception
    def getLatestGeneratedModelVersion(self, request, context):
        model_version_detail = self.store.get_latest_generated_model_version(request.name)
        if model_version_detail is None:
            model_version_relation = None
        else:
            model_relation = self.store.get_model_relation_by_name(request.name)
            model_version_relation = self.store.get_model_version_relation_by_version(
                model_version_detail.model_version,
                model_relation.uuid)
        return _wrap_meta_response(
            MetaToProto.model_version_store_to_proto(model_version_relation, model_version_detail))

    '''project api'''

    @catch_exception
    def getProjectById(self, request, context):
        project_meta = self.store.get_project_by_id(request.id)
        return _wrap_meta_response(MetaToProto.project_meta_to_proto(project_meta))

    @catch_exception
    def getProjectByName(self, request, context):
        project_meta = self.store.get_project_by_name(request.name)
        return _wrap_meta_response(MetaToProto.project_meta_to_proto(project_meta))

    @catch_exception
    def listProject(self, request, context):
        project_meta_list = self.store.list_project(request.page_size, request.offset)
        return _warp_project_list_response(project_meta_list)

    @catch_exception
    def registerProject(self, request, context):
        project = transform_project_meta(request.project)
        response = self.store.register_project(name=project.name, uri=project.uri, properties=project.properties)
        return _wrap_meta_response(MetaToProto.project_meta_to_proto(response))

    @catch_exception
    def updateProject(self, request, context):
        properties = None if request.properties == {} else request.properties
        project = self.store.update_project(project_name=request.name,
                                            uri=request.uri.value if request.HasField('uri') else None,
                                            properties=properties)
        return _wrap_meta_response(MetaToProto.project_meta_to_proto(project))

    @catch_exception
    def deleteProjectById(self, request, context):
        status = self.store.delete_project_by_id(request.id)
        return _wrap_delete_response(status)

    @catch_exception
    def deleteProjectByName(self, request, context):
        status = self.store.delete_project_by_name(request.name)
        return _wrap_delete_response(status)

    '''artifact api'''

    @catch_exception
    def getArtifactById(self, request, context):
        artifact_meta = self.store.get_artifact_by_id(request.id)
        return _wrap_meta_response(MetaToProto.artifact_meta_to_proto(artifact_meta))

    @catch_exception
    def getArtifactByName(self, request, context):
        artifact_meta = self.store.get_artifact_by_name(request.name)
        return _wrap_meta_response(MetaToProto.artifact_meta_to_proto(artifact_meta))

    @catch_exception
    def registerArtifact(self, request, context):
        artifact = transform_artifact_meta(request.artifact)
        response = self.store.register_artifact(name=artifact.name, artifact_type=artifact.artifact_type,
                                                description=artifact.description, uri=artifact.uri,
                                                properties=artifact.properties)
        return _wrap_meta_response(MetaToProto.artifact_meta_to_proto(response))

    @catch_exception
    def updateArtifact(self, request, context):
        properties = None if request.properties == {} else request.properties
        artifact = self.store.update_artifact(name=request.name,
                                              artifact_type=request.artifact_type.value if request.HasField(
                                                  'artifact_type') else None,
                                              properties=properties,
                                              description=request.description.value if request.HasField(
                                                  'description') else None,
                                              uri=request.uri.value if request.HasField('uri') else None,
                                              )
        return _wrap_meta_response(MetaToProto.artifact_meta_to_proto(artifact))

    @catch_exception
    def listArtifact(self, request, context):
        artifact_meta_list = self.store.list_artifact(request.page_size, request.offset)
        return _warp_artifact_list_response(artifact_meta_list)

    @catch_exception
    def deleteArtifactById(self, request, context):
        status = self.store.delete_artifact_by_id(request.id)
        return _wrap_delete_response(status)

    @catch_exception
    def deleteArtifactByName(self, request, context):
        status = self.store.delete_artifact_by_name(request.name)
        return _wrap_delete_response(status)

    @catch_exception
    def registerWorkflow(self, request, context):
        workflow = transform_workflow_meta(request.workflow)
        response = self.store.register_workflow(name=workflow.name,
                                                project_id=workflow.project_id,
                                                properties=workflow.properties)
        return _wrap_meta_response(MetaToProto.workflow_meta_to_proto(response))

    @catch_exception
    def updateWorkflow(self, request, context):
        properties = None if request.properties == {} else request.properties
        workflow = self.store.update_workflow(workflow_name=request.workflow_name,
                                              project_name=request.project_name,
                                              properties=properties)
        return _wrap_meta_response(MetaToProto.workflow_meta_to_proto(workflow))

    def getWorkflowById(self, request, context):
        workflow = self.store.get_workflow_by_id(workflow_id=request.id)
        return _wrap_meta_response(MetaToProto.workflow_meta_to_proto(workflow))

    def getWorkflowByName(self, request, context):
        workflow = self.store.get_workflow_by_name(project_name=request.project_name,
                                                   workflow_name=request.workflow_name)
        return _wrap_meta_response(MetaToProto.workflow_meta_to_proto(workflow))

    def deleteWorkflowById(self, request, context):
        status = self.store.delete_workflow_by_id(workflow_id=request.id)
        return _wrap_delete_response(status)

    def deleteWorkflowByName(self, request, context):
        status = self.store.delete_workflow_by_name(project_name=request.project_name,
                                                    workflow_name=request.workflow_name)
        return _wrap_delete_response(status)

    def listWorkflows(self, request, context):
        workflow_meta_list = self.store.list_workflows(project_name=request.project_name,
                                                       page_size=request.page_size,
                                                       offset=request.offset)
        return _wrap_workflow_list_response(workflow_meta_list)




# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
import time
from typing import Text, List, Optional, Dict

from ai_flow.plugin_interface.scheduler_interface import Scheduler, JobExecutionInfo, WorkflowExecutionInfo, \
    WorkflowInfo
from ai_flow.context.project_context import ProjectContext
from ai_flow.workflow.workflow import Workflow
from ai_flow.workflow.status import Status
from ai_flow_plugins.scheduler_plugins.airflow.dag_generator import DAGGenerator


class MockScheduler(Scheduler):

    def __init__(self, config: Dict):
        super().__init__(config)
        self.dag_generator = DAGGenerator()

    def submit_workflow(self, workflow: Workflow, project_context: ProjectContext, args: Dict = None) -> WorkflowInfo:
        code_text = self.dag_generator.generate(workflow=workflow,
                                                project_name=project_context.project_name)
        return WorkflowInfo(workflow_name=workflow.workflow_name, properties={'code': code_text})

    def delete_workflow(self, project_name: Text, workflow_name: Text) -> WorkflowInfo:
        return WorkflowInfo(workflow_name=workflow_name, namespace=project_name)

    def pause_workflow_scheduling(self, project_name: Text, workflow_name: Text) -> WorkflowInfo:
        return WorkflowInfo(workflow_name=workflow_name, namespace=project_name)

    def resume_workflow_scheduling(self, project_name: Text, workflow_name: Text) -> WorkflowInfo:
        return WorkflowInfo(workflow_name=workflow_name, namespace=project_name)

    def start_new_workflow_execution(self, project_name: Text, workflow_name: Text) -> Optional[WorkflowExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name=workflow_name, namespace=project_name)
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.RUNNING,
                                                        start_date=str(int(time.time()*1000)))
        return workflow_execution_info

    def stop_all_workflow_execution(self, project_name: Text, workflow_name: Text) -> List[WorkflowExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name=workflow_name, namespace=project_name)
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        return [workflow_execution_info]

    def stop_workflow_execution(self, execution_id: Text) -> Optional[WorkflowExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        return workflow_execution_info

    def get_workflow_execution(self, execution_id: Text) -> Optional[WorkflowExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)),
                                                        end_date=str(int(time.time() * 1000)))
        return workflow_execution_info

    def list_workflow_executions(self, project_name: Text, workflow_name: Text) -> List[WorkflowExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        return [workflow_execution_info]

    def start_job_execution(self, job_name: Text, execution_id: Text) -> JobExecutionInfo:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        job_execution_info: JobExecutionInfo = JobExecutionInfo(job_name=job_name,
                                                                status=Status.RUNNING,
                                                                workflow_execution=workflow_execution_info,
                                                                start_date=str(int(time.time() * 1000))
                                                                )
        return job_execution_info

    def stop_job_execution(self, job_name: Text, execution_id: Text) -> JobExecutionInfo:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        job_execution_info: JobExecutionInfo = JobExecutionInfo(job_name=job_name,
                                                                status=Status.KILLED,
                                                                workflow_execution=workflow_execution_info,
                                                                start_date=str(int(time.time() * 1000))
                                                                )
        return job_execution_info

    def restart_job_execution(self, job_name: Text, execution_id: Text) -> JobExecutionInfo:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        job_execution_info: JobExecutionInfo = JobExecutionInfo(job_name=job_name,
                                                                status=Status.RUNNING,
                                                                workflow_execution=workflow_execution_info,
                                                                start_date=str(int(time.time() * 1000))
                                                                )
        return job_execution_info

    def get_job_executions(self, job_name: Text, execution_id: Text) -> List[JobExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        job_execution_info: JobExecutionInfo = JobExecutionInfo(job_name=job_name,
                                                                status=Status.RUNNING,
                                                                workflow_execution=workflow_execution_info,
                                                                start_date=str(int(time.time() * 1000)),
                                                                end_date=str(int(time.time() * 1000))
                                                                )
        return [job_execution_info]

    def list_job_executions(self, execution_id: Text) -> List[JobExecutionInfo]:
        workflow_info = WorkflowInfo(workflow_name='workflow_name', namespace='project_name')
        workflow_execution_info = WorkflowExecutionInfo(workflow_execution_id='1',
                                                        workflow_info=workflow_info,
                                                        status=Status.KILLED,
                                                        start_date=str(int(time.time() * 1000)))
        job_execution_info: JobExecutionInfo = JobExecutionInfo(job_name='job_name',
                                                                status=Status.RUNNING,
                                                                workflow_execution=workflow_execution_info,
                                                                start_date=str(int(time.time() * 1000))
                                                                )
        return [job_execution_info]


# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from abc import abstractmethod
from typing import Dict, Text
from ai_flow.context.job_context import current_job_name
from pyflink.dataset import ExecutionEnvironment
from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import TableConfig, BatchTableEnvironment, StreamTableEnvironment, TableEnvironment, StatementSet


class FlinkEnv(object):
    """
    FlinkEnv is responsible for creating the objects(ExecutionEnvironment, TableEnvironment, StatementSet)
    needed to build a flink job.
    """

    @abstractmethod
    def create_env(self) -> (ExecutionEnvironment, TableEnvironment, StatementSet):
        pass


class FlinkBatchEnv(FlinkEnv):
    """
    FlinkBatchEnv is the default implementation of FlinkEnv, used in flink batch jobs.
    """

    def create_env(self) -> (ExecutionEnvironment, TableEnvironment, StatementSet):
        exec_env = ExecutionEnvironment.get_execution_environment()
        exec_env.set_parallelism(1)
        t_config = TableConfig()
        t_env = BatchTableEnvironment.create(exec_env, t_config)
        t_env.get_config().get_configuration().set_string("taskmanager.memory.task.off-heap.size", '80m')
        statement_set = t_env.create_statement_set()
        return exec_env, t_env, statement_set


class FlinkStreamEnv(FlinkEnv):
    """
    FlinkStreamEnv is the default implementation of FlinkEnv, used in flink streaming jobs.
    """

    def create_env(self) -> (ExecutionEnvironment, TableEnvironment, StatementSet):
        exec_env = StreamExecutionEnvironment.get_execution_environment()
        exec_env.set_parallelism(1)
        t_config = TableConfig()
        t_env = StreamTableEnvironment.create(exec_env, t_config)
        t_env.get_config().get_configuration().set_string("taskmanager.memory.task.off-heap.size", '80m')
        statement_set = t_env.create_statement_set()
        return exec_env, t_env, statement_set


__flink_env__: FlinkEnv = FlinkBatchEnv()

__job_flink_env_dict__: Dict[Text, FlinkEnv] = {}


def set_flink_env(env: FlinkEnv):
    global __flink_env__, __job_flink_env_dict__
    if current_job_name() is None:
        __flink_env__ = env
    else:
        __job_flink_env_dict__[current_job_name()] = env


def get_global_flink_env() -> FlinkEnv:
    global __flink_env__
    return __flink_env__


def get_flink_env_by_job_name(job_name: Text) -> FlinkEnv:
    global __job_flink_env_dict__
    return __job_flink_env_dict__.get(job_name)



# coding:utf-8
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
import os
import subprocess
from shutil import copytree, rmtree
from setuptools import setup, find_packages

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
in_source = os.path.isfile(CURRENT_DIR + "/run_tests.sh")


def remove_if_exists(file_path):
    if os.path.exists(file_path):
        if os.path.islink(file_path) or os.path.isfile(file_path):
            os.remove(file_path)
        else:
            assert os.path.isdir(file_path)
            rmtree(file_path)


def remove_installed_airflow():
    from distutils.sysconfig import get_python_lib

    local_site_package = get_python_lib()
    installed_airflow_path = os.path.join(local_site_package, "airflow")
    for file in os.listdir(installed_airflow_path):
        abs_path = os.path.join(installed_airflow_path, file)
        if os.path.isdir(abs_path) and file == 'providers':
            print("Airflow providers are not being removed.")
        else:
            remove_if_exists(abs_path)


def compile_assets():  # noqa
    # """Run a command to compile and build airflow assets."""
    subprocess.check_call('./lib/airflow/airflow/www/compile_assets.sh')


def get_script():
    bin_dir = os.path.join(CURRENT_DIR, "bin")
    return [os.path.join("bin", filename) for filename in os.listdir(bin_dir)]


try:
    if in_source:
        compile_assets()
        AIRFLOW_DIR = CURRENT_DIR + "/lib/airflow"
        try:
            os.symlink(AIRFLOW_DIR + "/airflow", CURRENT_DIR + "/airflow")
        except BaseException:  # pylint: disable=broad-except
            copytree(AIRFLOW_DIR + "/airflow", CURRENT_DIR + "/airflow")
    else:
        remove_installed_airflow()

    require_file = '{}/{}'.format(os.path.dirname(os.path.abspath(__file__)), "requirements.txt")
    with open(require_file) as f:
        context = f.read()
        require_file_lines = context.strip().split('\n')
    require_packages = []
    for line in require_file_lines:
        if not len(line.strip()) == 0 and not line.startswith("#"):
            require_packages.append(line)

    packages = find_packages()
    setup(
        name='ai_flow',
        version='0.1.0',
        description='An open source framework that bridges big data and AI.',
        author='',
        author_email='flink.aiflow@gmail.com',
        url='https://github.com/alibaba/flink-ai-extended',
        packages=find_packages(),
        install_requires=require_packages,
        python_requires='>=3.7, <3.8',
        include_package_data=True,
        scripts=get_script(),
    )
finally:
    if in_source:
        remove_if_exists(CURRENT_DIR + "/airflow")



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metric_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from . import message_pb2 as message__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='metric_service.proto',
  package='ai_flow',
  syntax='proto3',
  serialized_options=b'\n\020com.aiflow.protoZ\010/ai_flow\210\001\001\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14metric_service.proto\x12\x07\x61i_flow\x1a\x1cgoogle/api/annotations.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\rmessage.proto\"(\n\x11MetricNameRequest\x12\x13\n\x0bmetric_name\x18\x01 \x01(\t\"\x1b\n\x0bUuidRequest\x12\x0c\n\x04uuid\x18\x01 \x01(\x03\"B\n\x11MetricMetaRequest\x12-\n\x0bmetric_meta\x18\x01 \x01(\x0b\x32\x18.ai_flow.MetricMetaProto\"l\n\x12MetricMetaResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12-\n\x0bmetric_meta\x18\x03 \x01(\x0b\x32\x18.ai_flow.MetricMetaProto\"K\n\x14MetricSummaryRequest\x12\x33\n\x0emetric_summary\x18\x01 \x01(\x0b\x32\x1b.ai_flow.MetricSummaryProto\"u\n\x15MetricSummaryResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x33\n\x0emetric_summary\x18\x03 \x01(\x0b\x32\x1b.ai_flow.MetricSummaryProto\"i\n\x1dListDatasetMetricMetasRequest\x12\x14\n\x0c\x64\x61taset_name\x18\x01 \x01(\t\x12\x32\n\x0cproject_name\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\"e\n\x1bListModelMetricMetasRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x32\n\x0cproject_name\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\"r\n\x17ListMetricMetasResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12.\n\x0cmetric_metas\x18\x03 \x03(\x0b\x32\x18.ai_flow.MetricMetaProto\"\x96\x02\n\x1aListMetricSummariesRequest\x12\x31\n\x0bmetric_name\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x30\n\nmetric_key\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x33\n\rmodel_version\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12/\n\nstart_time\x18\x04 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\x12-\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.Int64Value\"}\n\x1bListMetricSummariesResponse\x12\x13\n\x0breturn_code\x18\x01 \x01(\x03\x12\x12\n\nreturn_msg\x18\x02 \x01(\t\x12\x35\n\x10metric_summaries\x18\x03 \x03(\x0b\x32\x1b.ai_flow.MetricSummaryProto2\x97\x0b\n\rMetricService\x12{\n\x12registerMetricMeta\x12\x1a.ai_flow.MetricMetaRequest\x1a\x1b.ai_flow.MetricMetaResponse\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metric/metric_meta/create:\x01*\x12y\n\x10updateMetricMeta\x12\x1a.ai_flow.MetricMetaRequest\x1a\x1b.ai_flow.MetricMetaResponse\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metric/metric_meta/update:\x01*\x12o\n\x10\x64\x65leteMetricMeta\x12\x1a.ai_flow.MetricNameRequest\x1a\x11.ai_flow.Response\",\x82\xd3\xe4\x93\x02&\"!/aiflow/metric/metric_meta/delete:\x01*\x12p\n\rgetMetricMeta\x12\x1a.ai_flow.MetricNameRequest\x1a\x1b.ai_flow.MetricMetaResponse\"&\x82\xd3\xe4\x93\x02 \x12\x1e/aiflow/metric/metric_meta/get\x12\x93\x01\n\x16listDatasetMetricMetas\x12&.ai_flow.ListDatasetMetricMetasRequest\x1a .ai_flow.ListMetricMetasResponse\"/\x82\xd3\xe4\x93\x02)\x12\'/aiflow/metric/metric_meta/dataset/list\x12\x8d\x01\n\x14listModelMetricMetas\x12$.ai_flow.ListModelMetricMetasRequest\x1a .ai_flow.ListMetricMetasResponse\"-\x82\xd3\xe4\x93\x02\'\x12%/aiflow/metric/metric_meta/model/list\x12\x87\x01\n\x15registerMetricSummary\x12\x1d.ai_flow.MetricSummaryRequest\x1a\x1e.ai_flow.MetricSummaryResponse\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metric/metric_summary/create:\x01*\x12\x85\x01\n\x13updateMetricSummary\x12\x1d.ai_flow.MetricSummaryRequest\x1a\x1e.ai_flow.MetricSummaryResponse\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metric/metric_summary/update:\x01*\x12o\n\x13\x64\x65leteMetricSummary\x12\x14.ai_flow.UuidRequest\x1a\x11.ai_flow.Response\"/\x82\xd3\xe4\x93\x02)\"$/aiflow/metric/metric_summary/delete:\x01*\x12s\n\x10getMetricSummary\x12\x14.ai_flow.UuidRequest\x1a\x1e.ai_flow.MetricSummaryResponse\")\x82\xd3\xe4\x93\x02#\x12!/aiflow/metric/metric_summary/get\x12\x8c\x01\n\x13listMetricSummaries\x12#.ai_flow.ListMetricSummariesRequest\x1a$.ai_flow.ListMetricSummariesResponse\"*\x82\xd3\xe4\x93\x02$\x12\"/aiflow/metric/metric_summary/listB\"\n\x10\x63om.aiflow.protoZ\x08/ai_flow\x88\x01\x01\x90\x01\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_api_dot_annotations__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,message__pb2.DESCRIPTOR,])




_METRICNAMEREQUEST = _descriptor.Descriptor(
  name='MetricNameRequest',
  full_name='ai_flow.MetricNameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_name', full_name='ai_flow.MetricNameRequest.metric_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=110,
  serialized_end=150,
)


_UUIDREQUEST = _descriptor.Descriptor(
  name='UuidRequest',
  full_name='ai_flow.UuidRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uuid', full_name='ai_flow.UuidRequest.uuid', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=152,
  serialized_end=179,
)


_METRICMETAREQUEST = _descriptor.Descriptor(
  name='MetricMetaRequest',
  full_name='ai_flow.MetricMetaRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_meta', full_name='ai_flow.MetricMetaRequest.metric_meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=181,
  serialized_end=247,
)


_METRICMETARESPONSE = _descriptor.Descriptor(
  name='MetricMetaResponse',
  full_name='ai_flow.MetricMetaResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.MetricMetaResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.MetricMetaResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_meta', full_name='ai_flow.MetricMetaResponse.metric_meta', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=249,
  serialized_end=357,
)


_METRICSUMMARYREQUEST = _descriptor.Descriptor(
  name='MetricSummaryRequest',
  full_name='ai_flow.MetricSummaryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_summary', full_name='ai_flow.MetricSummaryRequest.metric_summary', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=359,
  serialized_end=434,
)


_METRICSUMMARYRESPONSE = _descriptor.Descriptor(
  name='MetricSummaryResponse',
  full_name='ai_flow.MetricSummaryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.MetricSummaryResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.MetricSummaryResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_summary', full_name='ai_flow.MetricSummaryResponse.metric_summary', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=436,
  serialized_end=553,
)


_LISTDATASETMETRICMETASREQUEST = _descriptor.Descriptor(
  name='ListDatasetMetricMetasRequest',
  full_name='ai_flow.ListDatasetMetricMetasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset_name', full_name='ai_flow.ListDatasetMetricMetasRequest.dataset_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.ListDatasetMetricMetasRequest.project_name', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=555,
  serialized_end=660,
)


_LISTMODELMETRICMETASREQUEST = _descriptor.Descriptor(
  name='ListModelMetricMetasRequest',
  full_name='ai_flow.ListModelMetricMetasRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_name', full_name='ai_flow.ListModelMetricMetasRequest.model_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_name', full_name='ai_flow.ListModelMetricMetasRequest.project_name', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=662,
  serialized_end=763,
)


_LISTMETRICMETASRESPONSE = _descriptor.Descriptor(
  name='ListMetricMetasResponse',
  full_name='ai_flow.ListMetricMetasResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.ListMetricMetasResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.ListMetricMetasResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_metas', full_name='ai_flow.ListMetricMetasResponse.metric_metas', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=765,
  serialized_end=879,
)


_LISTMETRICSUMMARIESREQUEST = _descriptor.Descriptor(
  name='ListMetricSummariesRequest',
  full_name='ai_flow.ListMetricSummariesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric_name', full_name='ai_flow.ListMetricSummariesRequest.metric_name', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_key', full_name='ai_flow.ListMetricSummariesRequest.metric_key', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model_version', full_name='ai_flow.ListMetricSummariesRequest.model_version', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='ai_flow.ListMetricSummariesRequest.start_time', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='ai_flow.ListMetricSummariesRequest.end_time', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=882,
  serialized_end=1160,
)


_LISTMETRICSUMMARIESRESPONSE = _descriptor.Descriptor(
  name='ListMetricSummariesResponse',
  full_name='ai_flow.ListMetricSummariesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='return_code', full_name='ai_flow.ListMetricSummariesResponse.return_code', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_msg', full_name='ai_flow.ListMetricSummariesResponse.return_msg', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_summaries', full_name='ai_flow.ListMetricSummariesResponse.metric_summaries', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1162,
  serialized_end=1287,
)

_METRICMETAREQUEST.fields_by_name['metric_meta'].message_type = message__pb2._METRICMETAPROTO
_METRICMETARESPONSE.fields_by_name['metric_meta'].message_type = message__pb2._METRICMETAPROTO
_METRICSUMMARYREQUEST.fields_by_name['metric_summary'].message_type = message__pb2._METRICSUMMARYPROTO
_METRICSUMMARYRESPONSE.fields_by_name['metric_summary'].message_type = message__pb2._METRICSUMMARYPROTO
_LISTDATASETMETRICMETASREQUEST.fields_by_name['project_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMODELMETRICMETASREQUEST.fields_by_name['project_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICMETASRESPONSE.fields_by_name['metric_metas'].message_type = message__pb2._METRICMETAPROTO
_LISTMETRICSUMMARIESREQUEST.fields_by_name['metric_name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['metric_key'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['model_version'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['start_time'].message_type = google_dot_protobuf_dot_wrappers__pb2._INT64VALUE
_LISTMETRICSUMMARIESREQUEST.fields_by_name['end_time'].message_type = google_dot_protobuf_dot_wrappers__pb2._INT64VALUE
_LISTMETRICSUMMARIESRESPONSE.fields_by_name['metric_summaries'].message_type = message__pb2._METRICSUMMARYPROTO
DESCRIPTOR.message_types_by_name['MetricNameRequest'] = _METRICNAMEREQUEST
DESCRIPTOR.message_types_by_name['UuidRequest'] = _UUIDREQUEST
DESCRIPTOR.message_types_by_name['MetricMetaRequest'] = _METRICMETAREQUEST
DESCRIPTOR.message_types_by_name['MetricMetaResponse'] = _METRICMETARESPONSE
DESCRIPTOR.message_types_by_name['MetricSummaryRequest'] = _METRICSUMMARYREQUEST
DESCRIPTOR.message_types_by_name['MetricSummaryResponse'] = _METRICSUMMARYRESPONSE
DESCRIPTOR.message_types_by_name['ListDatasetMetricMetasRequest'] = _LISTDATASETMETRICMETASREQUEST
DESCRIPTOR.message_types_by_name['ListModelMetricMetasRequest'] = _LISTMODELMETRICMETASREQUEST
DESCRIPTOR.message_types_by_name['ListMetricMetasResponse'] = _LISTMETRICMETASRESPONSE
DESCRIPTOR.message_types_by_name['ListMetricSummariesRequest'] = _LISTMETRICSUMMARIESREQUEST
DESCRIPTOR.message_types_by_name['ListMetricSummariesResponse'] = _LISTMETRICSUMMARIESRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MetricNameRequest = _reflection.GeneratedProtocolMessageType('MetricNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICNAMEREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricNameRequest)
  })
_sym_db.RegisterMessage(MetricNameRequest)

UuidRequest = _reflection.GeneratedProtocolMessageType('UuidRequest', (_message.Message,), {
  'DESCRIPTOR' : _UUIDREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.UuidRequest)
  })
_sym_db.RegisterMessage(UuidRequest)

MetricMetaRequest = _reflection.GeneratedProtocolMessageType('MetricMetaRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICMETAREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricMetaRequest)
  })
_sym_db.RegisterMessage(MetricMetaRequest)

MetricMetaResponse = _reflection.GeneratedProtocolMessageType('MetricMetaResponse', (_message.Message,), {
  'DESCRIPTOR' : _METRICMETARESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricMetaResponse)
  })
_sym_db.RegisterMessage(MetricMetaResponse)

MetricSummaryRequest = _reflection.GeneratedProtocolMessageType('MetricSummaryRequest', (_message.Message,), {
  'DESCRIPTOR' : _METRICSUMMARYREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricSummaryRequest)
  })
_sym_db.RegisterMessage(MetricSummaryRequest)

MetricSummaryResponse = _reflection.GeneratedProtocolMessageType('MetricSummaryResponse', (_message.Message,), {
  'DESCRIPTOR' : _METRICSUMMARYRESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.MetricSummaryResponse)
  })
_sym_db.RegisterMessage(MetricSummaryResponse)

ListDatasetMetricMetasRequest = _reflection.GeneratedProtocolMessageType('ListDatasetMetricMetasRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTDATASETMETRICMETASREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListDatasetMetricMetasRequest)
  })
_sym_db.RegisterMessage(ListDatasetMetricMetasRequest)

ListModelMetricMetasRequest = _reflection.GeneratedProtocolMessageType('ListModelMetricMetasRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMODELMETRICMETASREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListModelMetricMetasRequest)
  })
_sym_db.RegisterMessage(ListModelMetricMetasRequest)

ListMetricMetasResponse = _reflection.GeneratedProtocolMessageType('ListMetricMetasResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETRICMETASRESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListMetricMetasResponse)
  })
_sym_db.RegisterMessage(ListMetricMetasResponse)

ListMetricSummariesRequest = _reflection.GeneratedProtocolMessageType('ListMetricSummariesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETRICSUMMARIESREQUEST,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListMetricSummariesRequest)
  })
_sym_db.RegisterMessage(ListMetricSummariesRequest)

ListMetricSummariesResponse = _reflection.GeneratedProtocolMessageType('ListMetricSummariesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTMETRICSUMMARIESRESPONSE,
  '__module__' : 'metric_service_pb2'
  # @@protoc_insertion_point(class_scope:ai_flow.ListMetricSummariesResponse)
  })
_sym_db.RegisterMessage(ListMetricSummariesResponse)


DESCRIPTOR._options = None

_METRICSERVICE = _descriptor.ServiceDescriptor(
  name='MetricService',
  full_name='ai_flow.MetricService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1290,
  serialized_end=2721,
  methods=[
  _descriptor.MethodDescriptor(
    name='registerMetricMeta',
    full_name='ai_flow.MetricService.registerMetricMeta',
    index=0,
    containing_service=None,
    input_type=_METRICMETAREQUEST,
    output_type=_METRICMETARESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metric/metric_meta/create:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateMetricMeta',
    full_name='ai_flow.MetricService.updateMetricMeta',
    index=1,
    containing_service=None,
    input_type=_METRICMETAREQUEST,
    output_type=_METRICMETARESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metric/metric_meta/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteMetricMeta',
    full_name='ai_flow.MetricService.deleteMetricMeta',
    index=2,
    containing_service=None,
    input_type=_METRICNAMEREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002&\"!/aiflow/metric/metric_meta/delete:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getMetricMeta',
    full_name='ai_flow.MetricService.getMetricMeta',
    index=3,
    containing_service=None,
    input_type=_METRICNAMEREQUEST,
    output_type=_METRICMETARESPONSE,
    serialized_options=b'\202\323\344\223\002 \022\036/aiflow/metric/metric_meta/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listDatasetMetricMetas',
    full_name='ai_flow.MetricService.listDatasetMetricMetas',
    index=4,
    containing_service=None,
    input_type=_LISTDATASETMETRICMETASREQUEST,
    output_type=_LISTMETRICMETASRESPONSE,
    serialized_options=b'\202\323\344\223\002)\022\'/aiflow/metric/metric_meta/dataset/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listModelMetricMetas',
    full_name='ai_flow.MetricService.listModelMetricMetas',
    index=5,
    containing_service=None,
    input_type=_LISTMODELMETRICMETASREQUEST,
    output_type=_LISTMETRICMETASRESPONSE,
    serialized_options=b'\202\323\344\223\002\'\022%/aiflow/metric/metric_meta/model/list',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='registerMetricSummary',
    full_name='ai_flow.MetricService.registerMetricSummary',
    index=6,
    containing_service=None,
    input_type=_METRICSUMMARYREQUEST,
    output_type=_METRICSUMMARYRESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metric/metric_summary/create:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='updateMetricSummary',
    full_name='ai_flow.MetricService.updateMetricSummary',
    index=7,
    containing_service=None,
    input_type=_METRICSUMMARYREQUEST,
    output_type=_METRICSUMMARYRESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metric/metric_summary/update:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='deleteMetricSummary',
    full_name='ai_flow.MetricService.deleteMetricSummary',
    index=8,
    containing_service=None,
    input_type=_UUIDREQUEST,
    output_type=message__pb2._RESPONSE,
    serialized_options=b'\202\323\344\223\002)\"$/aiflow/metric/metric_summary/delete:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='getMetricSummary',
    full_name='ai_flow.MetricService.getMetricSummary',
    index=9,
    containing_service=None,
    input_type=_UUIDREQUEST,
    output_type=_METRICSUMMARYRESPONSE,
    serialized_options=b'\202\323\344\223\002#\022!/aiflow/metric/metric_summary/get',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='listMetricSummaries',
    full_name='ai_flow.MetricService.listMetricSummaries',
    index=10,
    containing_service=None,
    input_type=_LISTMETRICSUMMARIESREQUEST,
    output_type=_LISTMETRICSUMMARIESRESPONSE,
    serialized_options=b'\202\323\344\223\002$\022\"/aiflow/metric/metric_summary/list',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_METRICSERVICE)

DESCRIPTOR.services_by_name['MetricService'] = _METRICSERVICE

MetricService = service_reflection.GeneratedServiceType('MetricService', (_service.Service,), dict(
  DESCRIPTOR = _METRICSERVICE,
  __module__ = 'metric_service_pb2'
  ))

MetricService_Stub = service_reflection.GeneratedServiceStubType('MetricService_Stub', (MetricService,), dict(
  DESCRIPTOR = _METRICSERVICE,
  __module__ = 'metric_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)
